@@ -23,8 +23,9 @@ class ScpiTcpIpTransport(ScpiTransportBase):
                  dnssd_services: List[str] = ('_scpi-raw._tcp.local.',),
                  dnssd_domains: List[str] = ('local',),
                  dnssd_names: List[str] = ('.*',),
+                 dnssd_ipversions: str = '',
                  dnssd_timeout: float = 0.5) -> List[ScpiResource]:
-        from zeroconf import Zeroconf, ServiceListener, ServiceBrowser
+        from zeroconf import Zeroconf, ServiceListener, ServiceBrowser, IPVersion
         import time
 
         class Listener(ServiceListener):
@@ -37,6 +38,7 @@ class ScpiTcpIpTransport(ScpiTransportBase):
             def add_service(self, zeroconf, zc_type, zc_name):
                 self.services.update({zc_name: zeroconf.get_service_info(zc_type, zc_name)})
 
+
             def update_service(self, zeroconf, zc_type, zc_name):
                 self.services.update({zc_name: zeroconf.get_service_info(zc_type, zc_name)})
 
@@ -46,8 +48,12 @@ class ScpiTcpIpTransport(ScpiTransportBase):
 
         # Find devices via zeroconf mDNS
         # TODO: Implement DNS-SD for non-.local domains
+        ip_version = IPVersion.All if '4' in dnssd_ipversions and '6' in dnssd_ipversions else \
+            IPVersion.V4Only if '4' in dnssd_ipversions else IPVersion.V6Only if '6' in dnssd_ipversions else None
+
         listener = Listener()
-        ServiceBrowser(Zeroconf(), dnssd_services_fq, listener=listener)
+        zc = Zeroconf(ip_version=ip_version)
+        ServiceBrowser(zc, dnssd_services_fq, listener=listener)
 
         # Wait for some time to get answers
         time.sleep(dnssd_timeout)
@@ -57,7 +63,7 @@ class ScpiTcpIpTransport(ScpiTransportBase):
 
         return [ScpiResource(transport=ScpiTcpIpTransport,
                              location=f'dnssd:{service.name}',
-                             address=(service.parsed_addresses()[0], int(service.port)),
+                             address=(parsed_address, int(service.port)),
                              name=service.get_name(),
                              manufacturer=service.properties[b'Manufacturer'].decode(
                                  'utf-8') if b'Manufacturer' in service.properties else None,
@@ -66,15 +72,15 @@ class ScpiTcpIpTransport(ScpiTransportBase):
                              serialnum=service.properties[b'SerialNumber'].decode(
                                  'utf-8') if b'SerialNumber' in service.properties else None,
                              info=service
-                             ) for service in listener.services.values()
+                             ) for service in listener.services.values() for parsed_address in service.parsed_scoped_addresses()
                 if any([pattern.fullmatch(service.get_name()) for pattern in patterns])]
 
     @classmethod
     def from_resource_name(cls, resource_name: str) -> Optional[ScpiResource]:
         m = re.match((
             r'^(?P<prefix>(?P<type>TCPIP)(?P<board>\d+)?)'
-            r'::(?P<host_address>.+)'
-            r'::(?P<port>([0-9]+))'
+            r'::((?P<host_address>[^:]+)|(\[(?P<host_address6>.+)\]))'
+            r'::(?P<host_port>([0-9]+))'
             r'(::(?P<suffix>SOCKET))?$'
         ), resource_name, re.IGNORECASE)
 
@@ -84,37 +90,46 @@ class ScpiTcpIpTransport(ScpiTransportBase):
 
         groupdict = m.groupdict()
 
+        # Get either IPv4 or IPv6 address
+        host_address = groupdict['host_address'] if groupdict['host_address'] is not None else groupdict['host_address6']
+        host_port = groupdict['host_port']
+
         return ScpiResource(
             transport=ScpiTcpIpTransport,
-            address=(groupdict["host_address"], int(groupdict["port"]))
+            address=(host_address, host_port)
         )
 
     @classmethod
     def to_resource_name(cls, resource: ScpiResource) -> str:
-        address = resource.address
-        return f'TCPIP::{address[0]}::{address[1]}::SOCKET'
+        host_address = resource.address[0] if not ':' in resource.address[0] else f'[{resource.address[0]}]'
+        host_port = resource.address[1] if len(resource.address) > 1 else cls.DEFAULT_PORT
 
-    def __init__(self, address: str, timeout: float = 5, **kwargs):
+        return f'TCPIP::{host_address}::{host_port}::SOCKET'
+
+    def __init__(self, address: Union[tuple, str], timeout: float = 5, **kwargs):
         super().__init__(**kwargs)
         self._logger = logging.getLogger(__name__)
-        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self._sock = None
 
-        self._sock.settimeout(timeout)
-        self._sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+        if isinstance(address, str):
+            # Convert from string to tuple first
+            addr = urlparse('//' + address)
+            if (addr.hostname is None):
+                raise ScpiTransportException(f'{address} is not a valid hostname')
+            address = (addr.hostname, addr.port) if addr.port is not None else (addr.hostname, self.DEFAULT_PORT)
 
-        # Get hostname and port
-        if isinstance(address, tuple):
-            # Convert from tuple to string first
-            address = f'{address[0]}:{address[1]}'
-
-        addr = urlparse('//' + address)
-        if (addr.hostname is None):
-            raise ScpiTransportException(f'{address} is not a valid hostname')
+        try:
+            # Get address info
+            addr_info = socket.getaddrinfo(address[0], address[1], type=socket.SOCK_STREAM, proto=socket.IPPROTO_TCP)
+        except socket.gaierror as e:
+            raise ScpiTransportException(e) from e
 
         try:
             # Try to connect
-            self._sock.connect((addr.hostname,
-                                addr.port if addr.port is not None else self.DEFAULT_PORT))
+            self._sock = socket.socket(addr_info[0][0], addr_info[0][1], addr_info[0][2])
+            self._sock.settimeout(timeout)
+            self._sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+            self._sock.connect(addr_info[0][4])
 
             # Make a file interface for easier handling
             self._io = self._sock.makefile('rwb', buffering=0)
